%%%%%%%%
%%% PREPARATION OF THE DATABASE
%%%%%%%%
close all; clear all;

%% Load quarterly data
% Command 'dbdload' loads the data from the 'csv' file (save from Excel as
% .csv in the current directory). All the data are now available in the
% database 'd' 
d = dbload('data.csv');

%% Seasonal adjustment
list = dbobjects(d);

for i = 1:length(list)
    if length(list{i})>1
        if strcmp('_u', list{i}(end-1:end))
            d.(list{i}(1:end-2)) = x12(d.(list{i}), Inf, 'mode', 'm');
            d = rmfield(d, list{i});
        end
    end
end

%% Make log of variables
exceptions = {'rn','x_rn'};

list = dbobjects(d);

for i = 1:length(list)
    if isempty(strmatch(list{i},exceptions,'exact'))
        d.(['l' list{i}]) = 100*log(d.(list{i}));
    end
end

%% Define the real exchange rate
d.lz = d.ls + d.lx_cpi - d.lcpi;

%% Growth rate qoq, yoy
exceptions = {'rn','x_rn'};

list = dbobjects(d);

for i = 1:length(list)
    if isempty(strmatch(list{i}, exceptions,'exact'))
        if length(list{i})>1
            if strcmp('l', list{i}(1:1))
                d.(['dot_' list{i}(2:end)])  = 4*(d.(list{i}) - d.(list{i}){-1});
                d.(['dot4_' list{i}(2:end)]) = d.(list{i}) - d.(list{i}){-4};
            end
        end
    end
end


%% Real variables
% Domestic real interest rate
d.rr = d.rn - d.dot4_cpi;

% Fooreign real interest rate
d.x_rr = d.x_rn - d.dot4_x_cpi;

%% Trends and Gaps - Hodrick-Prescott filter
list = {'rr','lz','x_rr'};

for i = 1:length(list)
    [d.([list{i} '_eq']), d.([list{i} '_gap'])] = hpf(d.(list{i}));
end

d.dot_z_eq = 4*(d.lz_eq - d.lz_eq{-1});

%% Trend and Gap for Output - Band-pass filter
d.lgdp_gap = bpass(d.lgdp,inf,[6,32],'detrend',false);
d.lgdp_eq = hpf((d.lgdp-d.lgdp_gap),inf,'lambda',5);
d.dot_gdp_eq = 4*(d.lgdp_eq - d.lgdp_eq{-1});

%% Foreign Output gap - Band-pass filter
d.lx_gdp_gap = bpass(d.lx_gdp,[6,32],inf,'detrend',false);

%% Inflation target (history)
d.target = hpf(d.dot_cpi);

% %If you want use a different number for the end value (there is 10 as a proxy) change the first lines 
%prior_cpi = tseries(get(d.dot_cpi,'end'),10);
% d.target = hpf(d.dot_cpi,inf,'level',prior_cpi);

%% Compute the exchange rate target over the history
%Exchange rate target is automatically calculated as to equal observed
%exchange rate at the end of the sample
prior_ls_tar = tseries(get(d.ls,'last'),d.ls(end));
d.ls_tar = hpf(d.ls,inf,'lambda',1600,'level',prior_ls_tar);
d.dot_s_tar = 4*(d.ls_tar - d.ls_tar{-1});

%% Expert change in the database--equilibrium real interest rate
d.shock_prem = tseries(get(d.ls,'range'),0);
% d.rr_eq(qq(2009,1)) = 1;

%% Expert change in the database--output gap
% d.lgdp_gap(qq(2008,3):qq(2009,1)) = 1;

%% Sets cumulative output gap zero at the end of the observed sample
d.cum_gap = tseries(get(d.lgdp,'first2last'),0);

%% Save the database
% Database is saved in file 'history.mat'
savestruct('history.mat',d);
dbsave(d,'history.csv');

%% Graphs
% Makes graphs of stylized facts
Plotrng = qq(1999,1):qq(2010,1);
set(0,'defaultlinelinewidth',2);

% Specify country
country = 'Bulgaria - Stylized Facts';

% Report
x = report('heading',country,'dateformat','YYYY:P','color', true);

x = newpage(x);

x = opengraph(x);

  x = tag(x,'Nominal Variables','fontsize','large');

  subplot(2,2,1);
  plot(Plotrng,d.dot_cpi,'b');
  hold on
  plot(Plotrng,d.dot4_cpi,'r');
  title('Inflation');
  legend('q-o-q','y-o-y','location','best');
  grid on; box off;
  
  subplot(2,2,2);
  plot(Plotrng,d.rn);
  title('Nominal Interest Rate');
  legend('Money Market Rate','location','best');
  grid on; box off;
  
  subplot(2,2,3);
  plot(Plotrng,d.dot_s);
  title('Nominal Exchange Rate (q-o-q)');
  grid on; box off;
  
  subplot(2,2,4);
  plot(Plotrng,d.dot_x_cpi);
  title('Foreign Inflation (q-o-q)'); 
  grid on; box off;
  
x = getgraph(x);

x = newpage(x);

x = opengraph(x);

  x = tag(x,'Real Variables and Trends','fontsize','large');

  subplot(3,2,1);
  plot(Plotrng,d.lgdp,'b');
  hold on
  plot(Plotrng,d.lgdp_eq,'r');
  title('Output'); 
  legend('Output','Trend','location','best');
  grid on; box off;
  
  subplot(3,2,2);
  plot(Plotrng,d.dot_gdp,'b');
  hold on
  plot(Plotrng,d.dot4_gdp,'r');
  title('GDP Growth'); 
  legend('q-o-q','y-o-y','location','best');
  grid on; box off;
  
  subplot(3,2,3);
  plot(Plotrng,d.rr,'b');
  hold on
  plot(Plotrng,d.rr_eq,'r');
  title('Real Interest Rate'); 
  legend('Real Interest Rate','Trend','location','best');
  grid on; box off;
  
  subplot(3,2,4);
  plot(Plotrng,d.x_rr,'b');
  hold on
  plot(Plotrng,d.x_rr_eq,'r');
  title('Foreign Real Interest Rate'); 
  legend('Foreign Real Interest Rate','Trend','location','best');
  grid on; box off;
  
  subplot(3,2,5);
  plot(Plotrng,d.lz,'b');
  hold on
  plot(Plotrng,d.lz_eq,'r');
  title('Real Exchange Rate'); 
  legend('Real Exchange','Trend','location','best');
  grid on; box off;
  
  subplot(3,2,6);
  plot(Plotrng,d.dot_z_eq,'r');
  title('Change in the Real Eq. Exchange Rate (q-o-q)'); 
  grid on; box off;
  
x = getgraph(x);

x = newpage(x);

x = opengraph(x);

  x = tag(x,'Gaps','fontsize','large');

  subplot(2,2,1); hold on;
  plot(Plotrng,d.lgdp_gap);
  title('Output Gap'); 
  grid on; box off;
  
  subplot(2,2,2); hold on;
  plot(Plotrng,d.rr_gap);
  title('Real Interest Rate Gap'); 
  grid on; box off;
  
  subplot(2,2,3); hold on;
  plot(Plotrng,d.lz_gap);
  title('Real Exchange Rate Gap'); 
  grid on; box off;
  
  subplot(2,2,4); hold on;
  plot(Plotrng,d.x_rr_gap);
  title('Foreign Real Interest Rate Gap'); 
  grid on; box off;
  
x = getgraph(x);

%% Saves all graphs
%% The default is set to generate both .ps and .pdf files:
compile(x,'Stylized_facts.ps','display',false);
compile(x,'Stylized_facts.pdf','display',false);